﻿/**
 * NewGame
 * Author: codelite
 * Description: new game instance
 */
package codelite
{
	/** flash classes **/
	import flash.display.Sprite;
	import flash.events.KeyboardEvent;
	import flash.events.Event;
	import flash.ui.Keyboard;
	import flash.utils.getTimer;
	import flash.media.Sound;
	import flash.media.SoundChannel;

	/** custom classes **/
	import codelite.utils.TextLabel;
	import codelite.utils.CustomGlow;
	import codelite.utils.Public;
	
	/** external classes **/
	import caurina.transitions.Tweener;
	import caurina.transitions.properties.ColorShortcuts;

	public class NewGame extends Sprite 
	{
		private var _scoreTxt		:TextLabel 		= null; 	// score text label
		private var _score   		:int 			= 0; 		// score points text label
		private var _shieldTxt		:TextLabel 		= null; 	// shield text label
		private var _cell	 		:MainCell  		= null;		// cell to be defend
		private var _keyRight		:Boolean		= false;	// if right key was pressed
		private var _keyLeft		:Boolean		= false;	// if left key was pressed
		private var _keyFire		:Boolean		= false;	// if fire key was pressed
		private var _nano			:NanoGfx		= null;		// nano ship instance
		private var _centerY		:Number			= 0;		// position x for cell
		private var _centerX		:Number			= 0;		// position y for cell
		private var _sin			:Number			= 0;		// sin value
		private var _cos			:Number			= 0;		// cos value
		private var _diameter		:Number			= 0;		// diameter of the cell
		private var _fireDelay		:Number			= 0;		// delay for fire 
		private var _lastShot		:Number			= 0;		// the time when the last projectile was fired
		private var _mArr			:Array			= null;		// missle array
		private var _eArr			:Array			= null;		// enemy array
		private var _expArr			:Array			= null;		// explosion array
		private var _music			:Sound 			= null;		// sound instance
		private var _musicChann 	:SoundChannel 	= null;		// sound channel instance
		private var _levelAnnouncer :TextLabel   	= null;		// level announcer label

		/** constructor **/
		public function NewGame()
		{
			// initialize caurina ColorShortcuts
			ColorShortcuts.init();
			
			// setup game elements
			Setup();

			// setup events nedded
			addEventListener(Event.ENTER_FRAME, Update,false,0,true);
			Public.ReturnStage().addEventListener(KeyboardEvent.KEY_DOWN, KeyDown,false,0,true);
			Public.ReturnStage().addEventListener(KeyboardEvent.KEY_UP, KeyUp,false,0,true);
			
			// set focus on this instance ( for keyboard interraction )
			Public.ReturnStage().focus = this;
			
			// sets the level of the game
			Public.SetLevel(1);
			
			// create the music
			_musicChann = new SoundChannel();
			_music = new Music();
			PlayMusic();
		}

		/** setups game elements  **/
		private function Setup():void 
		{
			// new cell instance
			_cell = new MainCell();
			addChild(_cell);
			
			// set the center of the stage
			_centerX=Public.ReturnStage().stageWidth/2;
			_centerY=Public.ReturnStage().stageHeight/2;
			
			_diameter=42;
			
			// new nano ship instance
			_nano= new NanoGfx();
			_nano.rotation = 90;
			addChild(_nano);
			
			_fireDelay=100;
		    _lastShot=0;
			
			// setup the nedeed arrays
			_mArr = [];
			_eArr = [];
			_expArr = [];
			
			// text for display score
			_scoreTxt = new TextLabel();
			_scoreTxt.SetFormat(35, 0xF68B1F, false, "left");
			_scoreTxt.SetText("score: 0");
			CustomGlow.Glow(_scoreTxt, 0x000000);
			_scoreTxt.x = 7;
			_scoreTxt.y = 3;
			_scoreTxt.alpha = 0.8;
			addChild(_scoreTxt);
			
			_score = 0;
			
			// text for display shield
			_shieldTxt = new TextLabel();
			_shieldTxt.SetFormat(35,0xF68B1F, false, "left");
			_shieldTxt.SetText("shield: 5");
			CustomGlow.Glow(_shieldTxt, 0x000000);
			_shieldTxt.x = Public.ReturnStage().stageWidth-_shieldTxt.width-3;
			_shieldTxt.y = 3;
			_shieldTxt.alpha = 0.8;
			addChild(_shieldTxt);
			
			// text for level announcer
			_levelAnnouncer = new TextLabel();
			_levelAnnouncer.SetFormat(35,0xFF6B00, false, "left");
			CustomGlow.Glow(_levelAnnouncer, 0x000000);
			ShowLevel("contamination level 1");
			
			// generate bad cells
			AddEnemiesToStage(5);

		}

		/** generates a new bad cell **/
		private function GenerateBadCell($pos:int=-1):void
		{
			if($pos == -1)
			{
				_eArr.push(new BadCell());
				addChildAt(_eArr[_eArr.length-1],0);
			}else{
				_eArr[$pos] = new BadCell();
				addChildAt(_eArr[$pos],0);
			}
		}
		
		/** adds bad cell to stage **/
		private function AddEnemiesToStage($nr:int):void
		{
			for ( var $a:int=0; $a<$nr; $a++)
			{
				GenerateBadCell(-1);
			}	
		}

		/** if keyboard key is pressed **/
		private function KeyDown($e:KeyboardEvent):void {
			switch ($e.keyCode) {
				case Keyboard.LEFT :
					_keyLeft=true;
					break;
				case Keyboard.RIGHT :
					_keyRight=true;
					break;
				case Keyboard.SPACE :
					_keyFire=true;
					break;
			}
		}

		/** if keyboard key is not pressed **/
		private function KeyUp($e:KeyboardEvent):void {
			switch ($e.keyCode) {
				case Keyboard.LEFT :
					_keyLeft=false;
					break;
				case Keyboard.RIGHT :
					_keyRight=false;
					break;
				case Keyboard.SPACE :
					_keyFire=false;
					break;
			}

		}
		
		/** updates the stage **/
		private function Update($e:Event):void {

			const $speed:Number=4;

			if (_keyLeft)
			{
				_sin+=.025;
				_cos+=.025;
				_nano.rotation -=5.725
			}

			if (_keyRight) 
			{
				_sin-=.025;
				_cos-=.025;
				_nano.rotation +=5.725
			}

			_nano.x=_centerX + Math.sin(_sin * $speed) * _diameter;
			_nano.y=_centerY + Math.cos(_cos *  $speed) * _diameter;

			if (_keyFire) FireBullet();
			
			MissleCollisionStage();
			MissleCollisionCell();
			BadCellCollision();
		}
		
		/** check missle - bad cell collision **/
		private function MissleCollisionCell():void
		{
			for(var $a:int =0; $a<_mArr.length; $a++)
			{
				for(var $b:int =0; $b<_eArr.length; $b++)
				{
					if(_eArr[$b] !=null && _mArr[$a]!=null)
					{
						if(_mArr[$a].hitTestObject(_eArr[$b]))
						{
							_eArr[$b].ShiledHit()
							RemoveBullet(_mArr[$a]);
							
							// this sets the new level once a score is reached
							
							if(_eArr[$b].GetShield() == 0)
							{
								AddExplosion(_eArr[$b].x,_eArr[$b].y);
								RemoveBadCell(_eArr[$b]);
								
								_score+=_eArr[$b].ReturnPoint();
								_scoreTxt.SetText("score: "+_score);
								
								if(Public.GetLevel() == 1)
								{
									if(_score >= 50)
									{
										Public.SetLevel(2);
										ShowLevel("contamination level 2");
										AddEnemiesToStage(1);
									}
								}else if(Public.GetLevel() == 2){
									if(_score >= 150)
									{
										Public.SetLevel(3);
										ShowLevel("contamination level 3");
									}
								}else if(Public.GetLevel() == 3){
									if(_score >= 300)
									{
										Public.SetLevel(4);
										ShowLevel("contamination level 4");
										AddEnemiesToStage(1);
										
									}
								}else if(Public.GetLevel() == 4){
									if(_score >= 550)
									{
										Public.SetLevel(5);
										ShowLevel("contamination level 5");
									}
								}else if(Public.GetLevel() == 5){
									if(_score >= 850)
									{
										Public.SetLevel(6);
										ShowLevel("contamination level 6");
										AddEnemiesToStage(1);
									}
								}else if(Public.GetLevel() == 6){
									if(_score >= 1250)
									{
										Public.SetLevel(7);
										ShowLevel("contamination level 7");
										AddEnemiesToStage(1);
									}
								}else if(Public.GetLevel() == 7){
									if(_score >= 1700)
									{
										Public.SetLevel(8);
										ShowLevel("contamination level 8");
										AddEnemiesToStage(1);
									}
								}else if(Public.GetLevel() == 8){
									if(_score >= 2200)
									{
										Public.SetLevel(9);
										ShowLevel("contamination level 9");
										AddEnemiesToStage(1);
									}
								}else if(Public.GetLevel() == 9){
									if(_score >= 2850)
									{
										Public.SetLevel(10);
										ShowLevel("contamination level 10");
										AddEnemiesToStage(1);
									}
								}else if(Public.GetLevel() == 10){
									if(_score >= 3300)
									{
										Public.SetLevel(11);
										ShowLevel("unknown virus detected !");
									}
								}
							}
						}
					}
				}
			}
		}
		
		/** check if missle leaves the stage **/		
		private function MissleCollisionStage():void
		{
			for(var $a:int =0; $a<_mArr.length; $a++)
			{
				if(_mArr[$a] !=null)
				{
					_mArr[$a].x += _mArr[$a].ReturnSpeedX();
					_mArr[$a].y += _mArr[$a].ReturnSpeedY();
					
					if (_mArr[$a].x >= Public.ReturnStage().stageWidth || 
			        	_mArr[$a].x <= 0 ||
						_mArr[$a].y > _mArr[$a].x >= Public.ReturnStage().stageHeight || 
						_mArr[$a].y<=0)
						{
							RemoveBullet(_mArr[$a]);
						}
				}
			}
		}
		
		/** check bad cell - main cell collision  **/				
		private function BadCellCollision():void
		{
			if(_eArr.length>0)
			{
				for(var $b:int =0; $b<_eArr.length;$b++)
				{
					if(_eArr[$b] != null)
					{
						var $i:Number = _cell.x - _eArr[$b].x;
			        	var $j:Number = _cell.y - _eArr[$b].y;
					
						var $mag:Number = Math.sqrt($i * $i + $j * $j);
						var $dif:Number = Math.min($mag, _eArr[$b].ReturnSpeed());	

						if(_eArr[$b].hitTestObject(_cell))
						{
							_cell.ShieldHit();
							Tweener.addTween(_cell, {_hue:180, time:0.3,
											 	onComplete:function():void
												{
													Tweener.addTween(_cell, {_hue:0, time:0.3})
												}
											 });
							RemoveBadCell(_eArr[$b]);
							
							_shieldTxt.SetText("shield: " + _cell.ReturnShield());
							
							if(_cell.ReturnShield() == 0)
							{
								Display(parent).GameOver(_score);
								return;
							}
							
						}else{
							$i /= $mag;
							$j /= $mag;
			
							$i *= $dif;
							$j *= $dif;
			
							_eArr[$b].x += $i;
							_eArr[$b].y += $j;
						}
					}
				}
			}
		}
		
		/** bullet is fired  **/				
		private function FireBullet():void
		{
			var $angle:Number=0;

			if (getTimer() > _lastShot + _fireDelay)
			{
				_mArr.push(new Bullet(0x000000,_nano.x,_nano.y));
				_mArr[_mArr.length-1].x=_nano.x;
				_mArr[_mArr.length-1].y=_nano.y;
				_lastShot=getTimer();
				$angle = ((_nano.rotation)*Math.PI/180);
				_mArr[_mArr.length-1].SetSpeedX(Math.cos($angle)*20);
				_mArr[_mArr.length-1].SetSpeedY(Math.sin($angle)*20);						
				addChild(_mArr[_mArr.length-1]);
			}
		}
		
		/** remove the bullet from memory  **/				
		private function  RemoveBullet($b:Bullet):void
		{
			for (var $a:int=0; $a< _mArr.length;$a++)
			{
				if ( _mArr[$a] == $b )
				{
					removeChild(_mArr[$a]);
					_mArr[$a].Clear();
					_mArr[$a] = null;
					_mArr.splice($a,1);
					break;
				}
			}
		}
				
		/** remove bad cell from memory  **/						
		private function  RemoveBadCell($c:BadCell):void
		{
			for (var $a:int=0; $a< _eArr.length; $a++)
			{
				if ( _eArr[$a] == $c )
				{
					removeChild(_eArr[$a]);
					$c.Clear();
					_eArr[$a] =null;
					GenerateBadCell($a);
					break;
				}
			}
		}
		
		/** adds explosion to stage  **/				
		private function AddExplosion($x:Number,$y:Number):void
		{
			_expArr.push(new Explosion(this));
			_expArr[_expArr.length-1].x =$x;
			_expArr[_expArr.length-1].y =$y;
			addChild(_expArr[_expArr.length-1]);
		}
		
		/** removes explosion from stage  **/				
		public function RemoveExplosion($o:Explosion):void
		{
			for(var $a:int =0; $a<_expArr.length;$a++)
			{
				if($o == _expArr[$a])
				{
					if(_expArr[$a])
					{
						if(_expArr[$a].stage) removeChild(_expArr[$a]);
						_expArr[$a].Clear();
						_expArr[$a]= null;
						_expArr.splice($a, 1);
						break;
					}
				}
			}
		}
		
		/** play the background music  **/				
		private function PlayMusic():void
		{
			_musicChann = _music.play(150);
			_musicChann.addEventListener(Event.SOUND_COMPLETE, LoopBeat, false, 0, true);
		}
		
		/** loop the music  **/				
		private function LoopBeat($e:Event):void
		{
			_musicChann.removeEventListener(Event.SOUND_COMPLETE, LoopBeat);
			PlayMusic();
		}
		
		/** shoe current level label **/
		private function ShowLevel($t:String):void
		{
			_levelAnnouncer.SetText($t)
			_levelAnnouncer.x = Public.ReturnStage().stageWidth/2 -_levelAnnouncer.width/2;
			_levelAnnouncer.y =60;
			_levelAnnouncer.alpha = 0;
			addChild(_levelAnnouncer);
			
			Tweener.addTween(_levelAnnouncer, {alpha:1, time:1, delay:1.5, transition:"easeOut.Cubic", 
							 onComplete:function():void
							 {
								 Tweener.addTween(_levelAnnouncer, {alpha:0, time:1, delay:2, transition:"easeOut.Cubic"});
							 }
							 });
		}
		
		/** destroy this from memory **/
		public function Clear():void
		{
			var $a:int =0;
			
			removeEventListener(Event.ENTER_FRAME, Update);
			Public.ReturnStage().removeEventListener(KeyboardEvent.KEY_DOWN, KeyDown);
			Public.ReturnStage().removeEventListener(KeyboardEvent.KEY_UP, KeyUp);
			_musicChann.removeEventListener(Event.SOUND_COMPLETE, LoopBeat);
			
			_musicChann.stop();
			_musicChann = null;
			_music		= null;
			
			if(_scoreTxt)
			{
				if(_scoreTxt.stage) removeChild(_scoreTxt);
				_scoreTxt.Clear();
				_scoreTxt = null;
			}
			
			_score = 0;

			if(_cell)
			{
				if(_cell.stage) removeChild(_cell);
				_cell.Clear();
				_cell = null;
			}
			
			if(_nano)
			{
				if(_nano.stage) removeChild(_nano)
				_nano = null;
			}
			
			for($a =0; $a<_mArr.length; $a++)
			{
				if(_mArr[$a])
				{
					if(_mArr[$a].stage) removeChild(_mArr[$a])
					_mArr[$a].Clear();
					_mArr[$a] = null;
				}
			}
			
			_mArr = null;
			
			
			for($a =0; $a<_eArr.length; $a++)
			{
				if(_eArr[$a])
				{
					if(_eArr[$a].stage) removeChild(_eArr[$a])
					_eArr[$a].Clear();
					_eArr[$a] = null;
				}
			}
			
			_eArr = null;
			
			
			for($a =0; $a<_expArr.length; $a++)
			{
				if(_expArr[$a])
				{
					if(_expArr[$a].stage) removeChild(_expArr[$a])
					_expArr[$a].Clear();
					_expArr[$a] = null;
				}
			}

			_expArr = null;
		}
	}
}